/*
 * SPGMR08 68HC08GP32 Loader
 * jb8 specific functions
 * Copyright (C) 2002, Anders Nystrom <andesr@southpole.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2.
 *  See the file COPYING that is distributed with this program.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
*/

#include <stdio.h>
#include <string.h>

#include "spgmr.h"
#include "mongp32.h"
#include "ops.h"
#include "lin_serial.h"
#include "s1utils.h"


struct limits {
	unsigned long flash_base;
	unsigned long flash_len;
	unsigned long vector_base;
	unsigned long vector_len;
};

static struct limits jb8_lim = {
	.flash_base =	0xDC00,
	.flash_len =	0x2000,
	.vector_base =	0xFFE0,
	.vector_len =	0x0020,
};


static int jb8_mass_block_erase(CPU *xcpu, int address)
{
	struct HC08stack stack;
	char c;

	if (load_module(xcpu, xcpu->erase_module) != 0)
		return -1;

	if (address != 0) {
		stack.a = 0x00;
		stack.h = (address>>8) & 0xFF;
		stack.x = address & 0xFF;
	} else {
		stack.a = 0x80;
		stack.h = 0xFF;
		stack.x = 0xFF;
	}
	stack.pc = 0x0090;
	mon_runstack(sfd, &stack);
	serial_read(sfd, &c, 1);
	xprint("Erased\n");
	return 0;
}

static int jb8_prog_vrfy_line(CPU *xcpu, char *buf,
		int address, int count, int op)
{
	int i, retry_cnt;
	int csum, tcsum;
	char echo[64];
	struct HC08stack stack;
	int rv;

	rv = 0;
	if (load_module(xcpu, xcpu->program_module) != 0)
		return -1;

	csum = 0;
	for (i = 0; i < count; i++)
		csum += buf[i];
	csum &= 0xFFFF;

	xprint("0x%04X\r", address & 0xFFFF, count);
	retry_cnt = 0;
	do {
		if (retry_cnt)
			xprint("0x%04X  %d retry\n", i, count);
		mon_blkwrite(sfd, buf, 0x004C, count);
		mon_blkread(sfd, echo, 0x004C, count);
		tcsum = 0;
		for (i = 0; i < count; i++)
			tcsum += echo[i];
		tcsum &= 0xFFFF;
	} while (csum != tcsum && !retry_cnt++);

	if (csum != tcsum) {
		eprint("data load error\n");
		mon_read_stack(sfd, &stack);
		mon_show_stack(sfd, &stack);
		xcpu->loaded_module = NULL;
		rv = -1;
	} else {
		stack.pc = 0x0100;
		stack.a = count+op;
		stack.h = (address>>8) & 0xFF;
		stack.x = address & 0xFF;
		mon_runstack(sfd, &stack);
		serial_read(sfd, echo, 1);
		if (op) {
			mon_read_stack(sfd, &stack);
			if (!(stack.cc & 0x01)) {
				eprint("verify error at 0x%02x%02x\n",
						stack.h, stack.x);
				mon_read_stack(sfd, &stack);
				mon_show_stack(sfd, &stack);
				xcpu->loaded_module = 0;
				rv =- 1;
			}
		}
	}

	return rv;
}

static int jb8_prog_vrfy(CPU *xcpu, char *filename, int op)
{
	FILE *fp;
	int i;
	int csum, tcsum;
	int rv;
	char data[MAXS1BYTES];
	unsigned long addr, next_addr;
	int len, maxlen;
	struct limits *lim;

	lim = (struct limits *)xcpu->cpu_data;
	rv = 0;

	if ( (fp = fopen(filename, "r")) == NULL) {
		xprint("cant open file %s\n", filename);
		return -1;
	}

	/* flash area */
	next_addr = lim->flash_base;
	while (rv == 0 && next_addr < lim->flash_base + lim->flash_len) {
		for (i = 0; i < sizeof(data); i++)
			data[i] = 0xFF;
		addr = next_addr;

		maxlen = sizeof(data);
		if (addr + maxlen > lim->flash_base + lim->flash_len)
			maxlen = lim->flash_base + lim->flash_len - addr;

		len = s1fsearchBLk(fp, data, addr, addr+maxlen-1);
		if (len < 0) {
			xprint("error while reading file %s\n", filename);
			rv = -1;
		} else if (len == 0) {
			next_addr = addr + maxlen;
		} else {
			next_addr = addr + len;
			rv = jb8_prog_vrfy_line(xcpu, data, addr, len, op);
		}
	}

	/* interrupt vectors */
	next_addr = lim->vector_base;
	while (rv == 0 && next_addr < lim->vector_base + lim->vector_len){
		for (i = 0; i < sizeof(data); i++)
			data[i] = 0xFF;
		addr = next_addr;

		maxlen = lim->vector_base + lim->vector_len - addr;

		len = s1fsearchBLk(fp, data, addr, addr+maxlen-1);
		if (len < 0) {
			xprint("error while reading file %s\n", filename);
			rv = -1;
		} else if (len == 0) {
			next_addr = addr + maxlen;
		} else {
			next_addr = addr + len;
			rv = jb8_prog_vrfy_line(xcpu, data, addr, len, op);
		}
	}

	fclose(fp);
	return rv;
}


static int jb8_erase(CPU *xcpu)
{
	return jb8_mass_block_erase(xcpu, 0);
}

static int jb8_program(CPU *xcpu, char *filename)
{
	return jb8_prog_vrfy(xcpu, filename, 0);
}

static int jb8_verify(CPU *xcpu, char *filename)
{
	return jb8_prog_vrfy(xcpu, filename, 128);
}

static int jb8_reset(CPU *xcpu, int which)
{
	unsigned char c;

	xcpu->loaded_module = NULL;

	mon_reset(sfd);
	if (mon_write_scode(sfd, which) < 0) {
		xprint("error writing scode\n");
		return -1;
	}

	if (mon_read(sfd, &c, 0x0040) < 0) {
		xprint("error reading reset status\n");
		return -1;
	}
	if ((c & 1<<6) == 0) {
		xprint("reset failed (0x0040 == %x)\n", c);
		return -1;
	}
	return 0;
}


static CPU jb8_cpu = {
	.cpu_name =		"jb8",
	.erase_function =	jb8_erase,
	.flash_function =	jb8_program,
	.verify_function =	jb8_verify,
	.reset_function =	jb8_reset,
	.verify_module =	LIBDIR"/jb8_loader.s19",
	.program_module =	LIBDIR"/jb8_loader.s19",
	.erase_module =		LIBDIR"/jb8_loader.s19",
	.cpu_data =		&jb8_lim,
};

CPU *jb8_init(CPU *xcpu)
{
	memcpy(xcpu, &jb8_cpu, sizeof(jb8_cpu));
	return xcpu;
};
