// $Id: lin_serial.c,v 1.7 2003/12/16 20:58:40 rwuest Exp $
/*
    SPGMR08 68HC08GP32 Loader
    Copyright (C) 2001  Sirius Engineering Company

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/serial.h>
#include <string.h>
#include "spgmr.h"
#include "lin_serial.h"

int dbgser = 0;
#ifndef DEBUG_SERIAL
	#define DBGSER(X)
#else
	#define DBGSER(X) X
#endif

struct termios oldser;	// place to save serial port termios settings
struct termios newser;	// place to work on serial port termios settings


struct xlate_baud {	// translateion of a standard numeric baud rate
	int num_value;			// to it's associated termios constant
	int termios_value;

} standardBaudRates[] = {
	{ 0,        B0},
	{ 50,       B50},       { 75,       B75},
	{ 110,      B110},      { 134,      B134},
	{ 150,      B150},      { 200,      B200},
	{ 300,      B300},      { 600,      B600},
	{ 1200,     B1200},     { 1800,     B1800},
	{ 2400,     B2400},     { 4800,     B4800},
	{ 9600,     B9600},     { 19200,    B19200},
	{ 38400,    B38400},    { 57600,    B57600},
	{ 115200,   B115200},   { 230400,   B230400}
};


/****************************************************************************/
/*                 int serial_open( char *portname )                        */
/*                                                                          */
/* will return -1 on failure                                                */
/* Returns the file descriptor                                              */
/****************************************************************************/
int serial_open( char *portname )
{
	int fd;
	int f;
	int baud_rate;

	
	fd = open( portname, O_RDWR | O_NDELAY  );	//open port | O_NOCTTY | O_NDELAY
	if ( fd == -1 ) {
		perror( "open failed" );
		return -1;
	}
/* Back to blocking mode. */
	f = fcntl( fd, F_GETFL, 0 );
	fcntl( fd, F_SETFL, f & ~O_NDELAY );
	
	if ( tcgetattr( fd, &oldser ) == -1 ) {	// get original serial settings
		perror("tcgetattr failed");
		return -1;
	}
	memcpy( &newser, &oldser, sizeof(struct termios) );	// copy original settings
	
	newser.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
	newser.c_lflag = 0;				// set input mode (non-canonical, no echo,...)
	newser.c_iflag = 0;				// also disables XON/XOFF control
	newser.c_oflag = 0;
	newser.c_cc[VTIME] = 2;		// tenths of a sec.
	newser.c_cc[VMIN] = 0;

	if ( tcsetattr( fd, TCSANOW, &newser ) == -1 ) {	// use the new termios settings
		perror( "tcsetattr(fd, ..) failed");
		return -1;
	}

	dtr_on( fd );

	return fd;
}

/****************************************************************************/
/*                 int serial_close( int fd )                               */
/*                                                                          */
/****************************************************************************/
int serial_close( int fd )
{
	close(fd);
	return 0;
}

/****************************************************************************/
/*                 int serial_reopen( char *portname )                      */
/*                                                                          */
/* will return -1 on failure                                                */
/* Returns the file descriptor                                              */
/****************************************************************************/
int serial_reopen( int ofd, char *portname )
{
	if ( (ofd != -1) && portname && *portname )		// be paranoid
		serial_close( ofd );

	if ( portname && *portname )
		return serial_open( portname );
	else
		return -1;

}

/****************************************************************************/
/*                 int serial_write( int fd, char *buf, int count)          */
/*                                                                          */
/****************************************************************************/
int serial_write( int fd, char *buf, int count)
{
DBGSER(
	int ii;
	if (debug && dbgser) {
		dprint( "serial_write, count = %d, data = ", count);
		for (ii=0; ii<count; ii++ )
			dprint( "0x%02X ", buf[ii] );
		dprint("\n");
	}
	)

	write( fd, buf, count );
	return 0;
}

/****************************************************************************/
/*                 int serial_read( int fd, char *buf, int count)           */
/*                                                                          */
/* returns the number of chars read                                         */
/****************************************************************************/
int serial_read( int fd, char *buf, int count)
{
	int cnt,jj;
DBGSER(
	int ii;
	if (debug&&dbgser) dprint( "serial_read, count = %d, ", count );
	)

	cnt = 0;
	for (jj=0; jj<count; jj++)
		cnt += read( fd, &buf[jj], 1 );
	
DBGSER(
	if (debug&&dbgser) {
		dprint( "read() = %d, data = ", cnt );
		for (ii=0; ii<cnt; ii++ )
			dprint( "0x%02X ", buf[ii] );
		dprint("\n");
	}
	)
	return cnt;
}

/****************************************************************************/
/*                 int serial_iflush( int fd )                              */
/*                                                                          */
/* Throw away any unread chars in the serail input buffer                   */
/****************************************************************************/
void serial_iflush(int fd)
{
	tcflush( fd, TCIFLUSH );
}

/*"One possibility jumped out at me.  You didn't incldue the code for
rts_off().  How did you code it?  If you just did a TIOCMSET, without
first doing an TIOCMGET (and then clearing the RTS flag, and then
calling TIOCMSET on the result), you probably disabled OUT2 on the
UART, which would have turned off interrupts on the receiver.  After
that, stuff would just accumulate in the UART's FIFO, and then when
you reset it, it would immediately get read out."  - Theodore Tso */

/****************************************************************************/
/*                 int rts_on(int fd)                                       */
/*                                                                          */
/* Force RTS signal true at file descriptor fd.                             */
/* Return 0 on success, errno on an error.                                  */
/****************************************************************************/
int rts_on(int fd)
{
	int ii = TIOCM_RTS;

DBGSER(
	if (debug&&dbgser) dprint( "called rts_on()\n" );
	)

	if( ioctl( fd, TIOCMBIS, &ii ) == -1) {	// RTS on
		perror("RTS on failed");
		return(errno);
	}
	return 0;
}

/****************************************************************************/
/*                 int rts_off(int fd)                                      */
/*                                                                          */
/* Force RTS signal false at file descriptor fd.                            */
/* Return 0 on success, errno on an error.                                  */
/****************************************************************************/
int rts_off(int fd)
{
	int ii = TIOCM_RTS;

DBGSER(
	if (debug&&dbgser) dprint( "called rts_off()\n" );
	)

	if( ioctl( fd, TIOCMBIC, &ii ) == -1) {	// RTS off
		perror("RTS off failed");
		return(errno);
	}
	return 0;
}

/****************************************************************************/
/*                 int dtr_on(int fd)                                       */
/*                                                                          */
/* Force DTR signal true at file descriptor fd.                             */
/* Return 0 on success, errno on an error.                                  */
/****************************************************************************/
int dtr_on(int fd)
{
	int ii = TIOCM_DTR;

DBGSER(
	if (debug&&dbgser) dprint( "called dtr_on()\n" );
	)

	if( ioctl( fd, TIOCMBIS, &ii ) == -1) {	// DTR on
		perror("DTR on failed");
		return(errno);
	}
	return 0;
}

/****************************************************************************/
/*                 int dtr_off(int fd)                                      */
/*                                                                          */
/* Force DTR signal false at file descriptor fd.                            */
/* Return 0 on success, errno on an error.                                  */
/****************************************************************************/
int dtr_off(int fd)
{
	int ii = TIOCM_DTR;

DBGSER(
	if (debug&&dbgser) dprint( "called dtr_off()\n" );
	)

	if( ioctl( fd, TIOCMBIC, &ii ) == -1) {	// DTR off
		perror("DTR off failed");
		return(errno);
	}
	return 0;
}
	
	
/****************************************************************************/
/*                 int serial_owait(int fd)                                 */
/*                                                                          */
/* Wait for transmitter to send all chars, including the complete last char */
/* sent by testiing the TSR register for empty.                             */
/* Returns 0 ion success, errno on an error.                                */
/* FIXME */
/****************************************************************************/
int serial_owait(int fd)	// wait for transmitter
{
	int aLSR;
	return (0);
	

	while(1)      // wait for empty TSR
	{
		if ( ioctl(fd,TIOCSERGETLSR,&aLSR) == -1 ) {
			perror( "GETLSR Failed" );
			return(errno);
		}
		if( aLSR ) break;		// TSR is empty
	}
	return 0;
}


/****************************************************************************/
/*                 int serial_status(int fd)                                */
/*                                                                          */
/* prints serial interrupt an i/o counts to stdout.                         */
/* Returns 0 on success, errno on failure                                   */
/****************************************************************************/
int serial_status(int fd)
{
	struct serial_icounter_struct icount;	// serial port interrupt counts

	if( ioctl( fd, TIOCGICOUNT, &icount ) == -1) {
		perror("TIOCGICOUNT failed");
		return(errno);
	}
	xprint( "rx = %d, tx = %d\n", icount.rx, icount.tx );
	xprint( "frame errs= %d, overruns= %d, parity= %d, brk= %d, buf_overruns= %d\n",
			icount.frame,
			icount.overrun,
			icount.parity,
			icount.brk,
			icount.buf_overrun
		);
	return 0;
}
	
/****************************************************************************/
/*                 int serial_baud(int fd, int rate)                        */
/*                                                                          */
/****************************************************************************/
int serial_set_termios_baud( int fd, int rate)
{
	int ii=0;
	struct termios tio;
	if ( tcgetattr( sfd, &tio ) == -1 ) {	// get original serial settings
		eprint("tcgetattr failed:\n");
		return -1;
	}
	cfsetispeed(&tio,rate);
	cfsetospeed(&tio,rate);
	if ( (ii = tcsetattr( fd, TCSANOW, &tio ) ) == -1 ) {
		eprint( "baud rate echange failed\n");
	}
	return ii;
}

/****************************************************************************/
/*                 int serial_setspeed(int fd, int rate)                    */
/*                                                                          */
/****************************************************************************/
int serial_setspeed( int fd, int rate)
{
	int ii;
	struct xlate_baud *xx;
	int cnt;
	/* find baud in array */
	/* so small a linear search will be reasonable */
	xx = standardBaudRates;
	cnt = sizeof(standardBaudRates)/sizeof(struct xlate_baud);
	for ( ii=0; ii<cnt; ii++ ) {
		if ( rate == standardBaudRates[ii].num_value )
			return serial_set_termios_baud( fd, standardBaudRates[ii].termios_value);
	}
	// if we got here it didn't match
	eprint(" Baud rate '%d' not support\n", baud_rate);
	return -1;
}







// $Log: lin_serial.c,v $
// Revision 1.7  2003/12/16 20:58:40  rwuest
// changed unsigned chars to char
// renamed baudrate to baud_rate
//
// Revision 1.6  2001/08/17 11:54:42  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.5  2001/08/11 14:36:49  rwuest
// set baudrate from global value
// rename "baudrate to rate in serial_setspeed() and
// serial_set_termios_baud() so I don;t get confused
//
// Revision 1.4  2001/08/04 16:59:52  rwuest
// require explicitly enabling dbgser to turn it on (not associated with the global int debug anymore
// Add baudrate changing
//
// Revision 1.3  2001/07/29 03:24:44  rwuest
// changed VTIME to 200 ms
//
// Revision 1.2  2001/07/22 00:28:31  rwuest
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
// serail debug messages now depend on both dbgser and debug
// added log messages at end
//
